<?php
/*************************
  Coppermine Photo Gallery
  ************************
  Copyright (c) 2003-2017 Coppermine Dev Team
  v1.0 originally written by Gregory Demar

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 3
  as published by the Free Software Foundation.

  ********************************************
  Coppermine version: 1.6.x
**********************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

function printUploadForm (&$upload_select, $pvals = array())
{
    global $CONFIG, $superCage, $lang_upload_php, $lang_common, $lang_plugin_url_upload, $icon_array;

    $vals = array_merge(array('title'=>'','caption'=>'','keywords'=>'','URI'=>''), $pvals);

    // hack so album select can be pre-selected
    if (isset($vals['url_album'])) { $superCage->post->_source['album'] = $vals['url_album']; }

    list($timestamp, $form_token) = getFormToken();

    $maxfilesizebytes = $CONFIG['max_upl_size'] * 1024;
    $maxfilesize = sprintf($lang_upload_php['max_fsize'], cpg_format_bytes($maxfilesizebytes));

    $upload_help = '&nbsp;<a class="greybox" href="plugins/url_upload/help.php?t=' . $CONFIG['theme'] . '&l='.$CONFIG['lang']
        . '&g=use" title="Help"><img src="images/help.gif" width="13" height="11" border="0" alt=""></a>';

    $upload_table_header = <<<EOT
    <table border="0" cellspacing="0" cellpadding="0" width="100%">
        <tr>
            <td>
                {$icon_array['upload']}{$lang_plugin_url_upload['title']} {$upload_help}
            </td>
            <td style="text-align:right">
                <span id="upload_method_selector">
                    {$upload_select}
                </span>
            </td>
        </tr>
    </table>
EOT;

    if (!(ini_get('allow_url_fopen') || extension_loaded('libcurl'))) {
        starttable('100%', $upload_table_header, 1);
        echo '<tr><td>';
        theme_msg_box('Sorry', $lang_plugin_url_upload['cant-use'], 'cpg_message_warning', '', '');
        echo '</td></tr>';
        endtable();
        return;
    }

    echo '<form method="post" action="" enctype="multipart/form-data" onsubmit="return url_form_check(this)">';
    starttable('100%', $upload_table_header, 2);
    echo '<tr><td colspan="2" class="tableb tableb_alternate"><strong>' . $maxfilesize . '</strong></td></tr>';
    form_alb_list_box($lang_common['album'], 'url_album');
    $keywordLabel = sprintf($lang_common['keywords_insert1'],$lang_common['keyword_separators'][$CONFIG['keyword_separator']]) . '<br /><a href="keyword_select.php" class="greybox">' . $lang_common['keywords_insert2'] .'</a>';

    echo <<<EOT
    <tr>
        <td width="40%" class="tableb">
            {$lang_plugin_url_upload['img-title']}
        </td>
        <td width="60%" class="tableb" valign="top">
            <input type="text" name="title" maxlength="255" value="{$vals['title']}" class="textinput" id="title" />
        </td>
    </tr>
    <tr>
        <td class="tableb" valign="top">
            {$lang_plugin_url_upload['img-desc']}&nbsp;<a class="greybox" href="help.php?f=empty.htm&amp;h=lang_bbcode_help_title&amp;t=lang_bbcode_help" rel="help.php?css=curve&amp;f=empty.htm&amp;h=lang_bbcode_help_title&amp;t=lang_bbcode_help" title="Hilfe"><img src="images/help.gif" width="13" height="11" border="0" alt="" /></a>
        </td>
        <td class="tableb" valign="top">
            <textarea name="caption" rows="5" cols="40" class="textinput">{$vals['caption']}</textarea>
        </td>
    </tr>
EOT;

    // add user fields if set in $CONFIG                                        // gmc - add user fields to plugin
    $hidden = '';                                                               // gmc - add user fields to plugin
    for ($i=1; $i<=4; $i++) {                                                   // gmc - add user fields to plugin
      if ($CONFIG["user_field{$i}_name"] != '') {                               // gmc - add user fields to plugin
        // user field set                                                       // gmc - add user fields to plugin
        echo <<<EOT
        <tr>
            <td width="40%" class="tableb">
               {$CONFIG["user_field{$i}_name"]}
            </td>
            <td width="60%" class="tableb" valign="top">
                <input type="text" name="user{$i}" maxlength="255" value='' class="textinput" id="keywords" />
            </td>
        </tr>
EOT;
      } else {                                                                  // gmc - add user fields to plugin
        $hidden .= "<input type=\"hidden\" name=\"user{$i}\" value=\"\" />";    // gmc - add user fields to plugin
      }                                                                         // gmc - add user fields to plugin
    }                                                                           // gmc - add user fields to plugin
                                                                                // gmc - add user fields to plugin (added $hidden to next echo)

    echo <<<EOT
    <tr>
        <td width="40%" class="tableb">
            {$keywordLabel}
        </td>
        <td width="60%" class="tableb" valign="top">
            <input type="text" name="keywords" maxlength="255" value="{$vals['keywords']}" class="textinput" id="keywords" />
        </td>
    </tr>
    <tr>
        <td width="40%" class="tableb">
            {$lang_plugin_url_upload['file-url']}
        </td>
        <td width="60%" class="tableb" valign="top">
            <input type="text" name="URI" maxlength="256" value="{$vals['URI']}" class="textinput" />
        </td>
    </tr>
    <tr>
        <td colspan="2">
            <input type="hidden" name="control" value="phase_1" />
            <input type="hidden" name="form_token" value="{$form_token}">
            <input type="hidden" name="timestamp" value="{$timestamp}">
            $hidden
        </td>
    </tr>
EOT;
    close_form($lang_plugin_url_upload['upload'].'&nbsp;', 1, 'ok');
    endtable();
    echo '</form>';
}


class URL_Getter
{
    protected $url, $ua, $mime = false, $ext, $size, $err = false;

    public function __construct ($url)
    {
        global $superCage;

        $purl = parse_url($url);
        if (!$purl) return false;
        if (!isset($purl['scheme'])) {
            $url = 'http://'.$url;
        }
        $this->url = $url;
        $this->ua = $superCage->server->getRaw('HTTP_USER_AGENT');
    }

    public function mime_type ()
    {
        if (!$this->mime) {
            $h = get_headers($this->url, 1);
            if (!(strstr($h[0], '200'))) {
                // The resource is not available. Generate the appropriate error.
                $this->err = $h[0];
                return false;
            }
            $this->mime = $h ? $h['Content-Type'] : false;
            $this->size = $h ? $h['Content-Length'] : false;
        }
        return $this->mime;
    }

    public function allowed (&$error)
    {
        global $CONFIG, $lang_plugin_url_upload, $max_file_size;
        $allowed_exts = array_merge(
            explode('/', $CONFIG['allowed_img_types']),
            explode('/', $CONFIG['allowed_mov_types']),
            explode('/', $CONFIG['allowed_snd_types']),
            explode('/', $CONFIG['allowed_doc_types'])
            );
        $mime = $this->mime_type();
        if (!$mime) {
            $error = $this->err ? sprintf($lang_plugin_url_upload['hdr_err'], $this->err) : 'Could not get data type';
            return false;
        }
        if ($this->size > $max_file_size) {
            $error = sprintf($lang_plugin_url_upload['too_big'], cpg_format_bytes($this->size));
            return false;
        }
        $result = cpg_db_query("SELECT extension FROM {$CONFIG['TABLE_FILETYPES']} WHERE mime='$mime'");
        $ext = cpg_db_fetch_assoc($result);
        cpg_db_free_result($result);
        if ($ext && in_array($ext['extension'], $allowed_exts)) {
            $this->ext = $ext['extension'];
            return true;
        }
        $error = sprintf($lang_plugin_url_upload['na_type'], $mime);
        return false;
    }

    public function upload ($album, $title, $caption, $keywords, &$error)
    {
        global $CONFIG, $lang_db_input_php, $superCage;
        do {
            // Check for valid album
            $q = "SELECT category FROM {$CONFIG['TABLE_ALBUMS']} WHERE aid='$album'";
            if (!GALLERY_ADMIN_MODE) {
                $q .= ' and (uploads = \'YES\' OR category = \''.(USER_ID + FIRST_USER_CAT).'\' OR owner = \''.USER_ID.'\')';
            }
            $result = cpg_db_query($q);
            if (!$result) {
                $error = $lang_db_input_php['unknown_album'];
                break;
            }
            cpg_db_free_result($result);

            // Form an acceptable filename
            $parts = parse_url($this->url);
            $file = basename($parts['path']);
            $file = CPGPluginAPI::filter('upload_file_name', $file);
            $file = replace_forbidden($file);

            // Create a unique temp file name
            $prefix = './'.$CONFIG['fullpath'].'edit/URL_tmp_'.USER_ID.'_'.$album.'_';
            $seed = time();
            do {
                ++$seed;
                $tempath = $prefix.$seed.'.'.$this->ext;
            } while (file_exists($tempath));

            // Get the URL data
            $content = file_get_contents($this->url);
            if (!$content) {
                $error = 'no_data';
                break;
            }
            // Write it to the temp file
            if (file_put_contents($tempath, $content) === false) {
                $error = 'no_save';
                break;
            }

            // Change file permission
            @chmod($tempath, octdec($CONFIG['default_file_mode'])); //silence the output in case chmod is disabled

            // Pictures are moved in a directory named 10000 + USER_ID
            if (USER_ID && $CONFIG['silly_safe_mode'] != 1) {
                $filepath = $CONFIG['userpics'] . (USER_ID + FIRST_USER_CAT);
                $dest_dir = $CONFIG['fullpath'] . $filepath;
                if (!is_dir($dest_dir)) {
                    mkdir($dest_dir, octdec($CONFIG['default_dir_mode']));
                    if (!is_dir($dest_dir)) {
                        @unlink($tempath);
                        $error = sprintf($lang_db_input_php['err_mkdir'], $dest_dir);
                        break;
                    }
                    @chmod($dest_dir, octdec($CONFIG['default_dir_mode'])); //silence the output in case chmod is disabled
                    $fp = fopen($dest_dir . '/index.php', 'w');
                    fwrite($fp, ' ');
                    fclose($fp);
                }
                // Upload pictures in a sub-directory named according to the album ID
                if ($CONFIG['upload_create_album_directory']) {
                    $filepath .= '/'.$album;
                    $dest_dir .= '/'.$album;
                    if (!is_dir($dest_dir)) {
                        mkdir($dest_dir, octdec($CONFIG['default_dir_mode']));
                        if (!is_dir($dest_dir)) {
                            @unlink($tempath);
                            $error = sprintf($lang_db_input_php['err_mkdir'], $dest_dir);
                            break;
                        }
                        @chmod($dest_dir, octdec($CONFIG['default_dir_mode'])); //silence the output in case chmod is disabled
                        $fp = fopen($dest_dir . '/index.php', 'w');
                        fwrite($fp, ' ');
                        fclose($fp);
                    }
                }
                $dest_dir .= '/';
                $filepath .= '/';
            } else {
                $filepath = $CONFIG['userpics'];
                $dest_dir = $CONFIG['fullpath'] . $filepath;
            }

            // Check that target dir is writable
            if (!is_writable($dest_dir)) {
                @unlink($tempath);
                $error = sprintf($lang_db_input_php['dest_dir_ro'], $dest_dir);
                break;
            }

            // Create a unique name for the uploaded file
            preg_match('/(.+)\.(.*?)\Z/', $file, $matches);
            $nr = 0;
            $file_name = $matches[1] . '.' . $matches[2];
            while (file_exists($dest_dir . $file_name)) {
                $file_name = $matches[1] . '~' . $nr++ . '.' . $matches[2];
            }
            $dest_file = $dest_dir . $file_name;


            // Set config for auto-orientation if requested
            if ($superCage->post->getInt('autorient')) {
                $CONFIG['autorient'] = 1;
            }

            // Move the picture into its final location
            if (rename($tempath, $dest_file)) {
                // Create thumbnail and intermediate image and add the image into the DB
                $user1 = $superCage->post->getEscaped('user1');                 // gmc - add user fields to plugin
                $user2 = $superCage->post->getEscaped('user2');                 // gmc - add user fields to plugin
                $user3 = $superCage->post->getEscaped('user3');                 // gmc - add user fields to plugin
                $user4 = $superCage->post->getEscaped('user4');                 // gmc - add user fields to plugin
//                if (add_picture($album, $filepath, $file_name, 0, $title, $caption, $keywords)) {   //, $user1, $user2, $user3, $user4, $category) {  // gmc - add user fields to plugin
                if (add_picture($album, $filepath, $file_name, 0, $title, $caption, $keywords, $user1, $user2, $user3, $user4)) {   //, $category) {  // gmc - add user fields to plugin
                    break;
                } else {
                    @unlink($dest_file);
                    $error = 'no-addb';
                    break;
                }
            } else {
                @unlink($tempath);
                $error = 'no-move';
                break;
            }
            
        } while (0);
    }

    private function get_url_data ()
    {
    }

}